precision mediump float;
varying vec2 v_Tex;
uniform sampler2D u_Tex;
uniform float intensity;

const vec3 luminanceWeighting = vec3(0.2125, 0.7154, 0.0721);

vec4 shadowAdjust(vec4 col, float shadows) {

    float luminance = dot(col.rgb, luminanceWeighting);
    float highlights = 1.0;

    //(shadows+1.0) changed to just shadows:
    float shadow = clamp((pow(luminance, 1.0/shadows) + (-0.76)*pow(luminance, 2.0/shadows)) - luminance, 0.0, 1.0);
    float highlight = clamp((1.0 - (pow(1.0-luminance, 1.0/(2.0-highlights)) + (-0.8)*pow(1.0-luminance, 2.0/(2.0-highlights)))) - luminance, -1.0, 0.0);
    vec3 result = ((luminance + shadow + highlight)) * ((col.rgb)/(luminance));

    // blend toward white if highlights is more than 1
    float contrastedLuminance = ((luminance - 0.5) * 1.5) + 0.5;
    float whiteInterp = pow(contrastedLuminance, 3.0);
    float whiteTarget = clamp(highlights, 1.0, 2.0) - 1.0;
    result = mix(result, vec3(1.0), whiteInterp*whiteTarget);

    // blend toward black if shadows is less than 1
    float invContrastedLuminance = 1.0 - contrastedLuminance;
    float blackInterp = pow(invContrastedLuminance, 3.0);
    float blackTarget = 1.0 - clamp(shadows, 0.0, 1.0);
    result = mix(result, vec3(0), blackInterp * blackTarget);

    return vec4(result, col.a);

}

void main(void) {

    float intensity = intensity + 0.5;
    vec4 color = texture2D(u_Tex, v_Tex);

    color = shadowAdjust(color, intensity);
    gl_FragColor = color;

}
