precision mediump float;
varying vec2 v_Tex;
uniform sampler2D u_Tex;
uniform float intensity;
uniform vec2 effectTextureSize;

vec4 gaussianBlur3(sampler2D image, vec2 uv, vec2 direction) {
    vec3 color = vec3(0.0);
    vec2 off1 = vec2(1.411764705882353) * direction;
    vec2 off2 = vec2(3.2941176470588234) * direction;
    vec2 off3 = vec2(5.176470588235294) * direction;
    color += texture2D(image, uv).rgb * 0.1964825501511404;
    color += texture2D(u_Tex, uv + off1).rgb * 0.2969069646728344;
    color += texture2D(u_Tex, uv - off1).rgb * 0.2969069646728344;
    color += texture2D(u_Tex, uv + off2).rgb * 0.09447039785044732;
    color += texture2D(u_Tex, uv - off2).rgb * 0.09447039785044732;
    color += texture2D(u_Tex, uv + off3).rgb * 0.010381362401148057;
    color += texture2D(u_Tex, uv - off3).rgb * 0.010381362401148057;
    return vec4(color, 1.0);
}

void main(void) {

    vec4 color = texture2D(u_Tex, v_Tex);
    float intensity = mix(-1.5, 1.5, intensity);

    if(intensity < 0.0) {
        vec2 direction = vec2(-intensity) / effectTextureSize;
        gl_FragColor = gaussianBlur3(u_Tex, v_Tex, direction);
    } else if(intensity > 0.0) {
    
        float dx = intensity;
        float dy = intensity;
        vec4 sum = vec4(0.0);
        sum += -1. * texture2D(u_Tex, v_Tex + vec2( -1.0 * dx , 0.0 * dy) / effectTextureSize);
        sum += -1. * texture2D(u_Tex, v_Tex + vec2( 0.0 * dx , -1.0 * dy) / effectTextureSize);
        sum +=  5. * texture2D(u_Tex, v_Tex + vec2( 0.0 * dx , 0.0 * dy) / effectTextureSize);
        sum += -1. * texture2D(u_Tex, v_Tex + vec2( 0.0 * dx , 1.0 * dy) / effectTextureSize);
        sum += -1. * texture2D(u_Tex, v_Tex + vec2( 1.0 * dx , 0.0 * dy) / effectTextureSize);
        gl_FragColor = (clamp(sum, vec4(0.0), vec4(1.0)));
    
    } else {
        
        gl_FragColor = texture2D(u_Tex, v_Tex);

    }

}
