precision mediump float;
varying vec2 v_Tex;
uniform sampler2D u_Tex;
uniform float intensity;
uniform float radius;

// Multiply

float blendMultiply(float background, float foreground) {
    return background * foreground;
}

vec3 blendMultiply(vec3 background, vec3 foreground) {
    return vec3(blendMultiply(background.r, foreground.r),
    blendMultiply(background.g, foreground.g),
    blendMultiply(background.b, foreground.b));
}

vec3 blendMultiply(vec3 background, vec3 foreground, float opacity) {
    return vec3(blendMultiply(background, foreground) * opacity + background * (1.0 - opacity));
}

vec4 blendMultiply(vec4 background, vec4 foreground) {
    return vec4(blendMultiply(background.rgb, foreground.rgb, foreground.a),
    min(background.a + foreground.a, 1.0));
}

float cutCircle(vec2 uv, vec2 center, float radius, float tolerance) {
    vec2 dist = center - uv;
    return 1.0 - smoothstep(radius - tolerance,
                            radius + tolerance,
                            length(dist));
}

float cutCircle(float ratio, vec2 uv, vec2 center, float radius, float tolerance) {
    uv.x *= ratio;
    center.x *= ratio;
    vec2 dist = center - uv;
    return 1.0 - smoothstep(radius - tolerance,
                            radius + tolerance,
                            length(dist));
}

void main(void) {

    vec4 col = texture2D(u_Tex, v_Tex);

    float r = mix(0.35, 0.65, radius);

    vec2 center = vec2(0.5);
    float tolerance = 0.5;

    float circle = cutCircle(v_Tex, center, r, tolerance);

    if(intensity <= 0.5) {
        float t = intensity / 0.5;
        col.rgb = mix(col.rgb / (circle + 0.25), col.rgb, t);
    } else {
        float t = pow((intensity - 0.5) / 0.5, 1.0/4.0);
        col.rgb = blendMultiply(col.rgb, vec3(circle), (intensity - 0.5) / 0.5);
    }

    gl_FragColor = col;

}