precision highp float;

varying vec2 v_Tex;
uniform sampler2D u_Tex;
uniform float timePoint;
uniform float intensity; // 0 to 1 value
uniform float resolution; // 0 to 1 value
uniform vec2 effectTextureSize;

// Blend Overlay

float blendOverlay(float background, float foreground) {
    float result = background <= 0.5 ? (2.0 * background * foreground) :
    (1.0 - (2.0 * (1.0 - background) * (1.0 - foreground)));
    return result;
}

vec3 blendOverlay(vec3 background, vec3 foreground) {
    return vec3(blendOverlay(background.r, foreground.r),
                blendOverlay(background.g, foreground.g),
                blendOverlay(background.b, foreground.b));
}

vec3 blendOverlay(vec3 background, vec3 foreground, float opacity) {
    return vec3(blendOverlay(background, foreground) * opacity + background * (1.0 - opacity));
}

vec4 blendOverlay(vec4 background, vec4 foreground) {
    return vec4(blendOverlay(background.rgb, foreground.rgb, foreground.a),
                min(background.a + foreground.a, 1.0));
}

float rand(vec2 co) {
    return fract(sin(dot(co.xy, vec2(12.9898,78.233))) * 43758.5453);
}

// Contrast

vec3 adjustContrast(vec3 color, float contrast) {
    float t = 0.5 - contrast * 0.5;
    return clamp(color * vec3(contrast) + vec3(t), 0.0, 1.0);
}

vec4 adjustContrast(vec4 color, float contrast) {
    return vec4(adjustContrast(color.rgb, contrast), color.a);
}

void main() {

    vec4 imageCol = texture2D(u_Tex, v_Tex);
    float res = mix(300.0, max(effectTextureSize.x, effectTextureSize.y), resolution);

    vec2 uv = v_Tex;
    uv = floor(uv * res + 0.5) / res;
    uv *= effectTextureSize.x / effectTextureSize.y;
    vec4 grain = vec4(
        rand(uv + mod(timePoint, 3.0)),
        rand(uv - 0.125 + mod(timePoint, 3.0)),
        rand(uv + 0.234 + mod(timePoint, 3.0)),
        1.0
    );

    grain = adjustContrast(grain, intensity);

    gl_FragColor = blendOverlay(imageCol, grain);

}
