precision highp float;

varying vec2 v_Tex;
uniform sampler2D u_Tex;

#define AVERAGE 1
#define LIGHTNESS 2
#define LUMINOSITY 4

vec3 grayscale(vec3 color, int grayMode) {
    float red = color.r;
    float green = color.g;
    float blue = color.b;

    float conversion;

    if(grayMode == AVERAGE) { // average method
        conversion = (red + green + blue) / 3.0;
    } else if(grayMode == LIGHTNESS) {// lightness method
        conversion = (max(red, max(green, blue)) + min(red, min(green, blue))) / 2.0;
    } else if(grayMode == LUMINOSITY) {// luminosity method
        conversion = (0.2125 * red) + (0.7154 * green) + (0.0721 * blue);
    }

    return vec3(conversion);
}

vec4 grayscale(vec4 color, int grayMode) {
    return vec4(grayscale(color.rgb, grayMode), color.a);
}

float sdRect(float ratio, vec2 uv, vec2 center, vec2 dim, float tolerance) {
    uv.x *= ratio;
    center.x *= ratio;
    vec2 p = uv - center;

    vec2 d = abs(p)-abs(dim);
    return 1.0 - smoothstep(-tolerance,
    tolerance,
    length(max(d,0.0)) + min(max(d.x,d.y),0.0));
}

void main(void) {

    vec2 uv = v_Tex;

    float f = 0.999;

    uv *= 3.999;
    uv = f * (uv - 0.5) + 0.5;

    int x = int(uv.x);
    int y = int(4. - uv.y);

    int index = x + y * 4;

    int colIndex = y > 1 ? index + y : 7 - index + y;

    if(int(mod(float(colIndex), 2.)) == 0) uv = 1. - uv;
    uv = fract(uv);
    uv = clamp(uv, 0.0, 1.0);
    uv = f * (uv - 0.5) + 0.5;

    vec4 col = texture2D(u_Tex, uv);

    vec4 color[4];

    vec4 color1 = vec4(0.973, 0.800, 0.902, 1.0);
    vec4 color2 = vec4(0.929, 0.608, 0.796, 1.0);
    vec4 color3 = vec4(0.871, 0.439, 0.690, 1.0);
    vec4 color4 = vec4(0.804, 0.302, 0.596, 1.0);

    if(y > 2) {
        color[0] = color1;
        color[3] = color2;
        color[2] = color3;
        color[1] = color4;
    } else if(y > 1) {
        color[0] = color1;
        color[1] = color2;
        color[2] = color3;
        color[3] = color4;
    } else if(y > 0) {
        color[0] = color1;
        color[1] = color1;
        color[2] = color3;
        color[3] = color2;
    } else {
        color[0] = color3;
        color[1] = color1;
        color[2] = color4;
        color[3] = color2;
    }

    vec4 gray = grayscale(col, LUMINOSITY);

    col = gray * color[int(mod(float(colIndex), 4.))];

    color[0] = color1;
    color[1] = color2;
    color[2] = color3;
    color[3] = color4;

    vec2 dim = vec2(0.785 / 2., 0.26 / 2.);
    vec2 center = 1. - vec2(dim.x, -dim.y) - vec2(0, 1. - 0.25);
    float tol = 0.0000001;
    float rect = sdRect(1.0, v_Tex, center, dim, tol);

    float scale = 3.825;
    uv = (v_Tex - vec2(1., 0.)) * scale + 1.;
    uv += vec2(2., -(1. / 4.) * scale);
    uv = f * (uv - 0.5f) + 0.5;
    index = int(uv.x) + int(1. - uv.y) * 4;
    colIndex = index + int(1. - uv.y) + 3;
    if(int(mod(float(colIndex), 2.)) == 0) uv = 1. - uv;
    uv = fract(uv);
    uv = clamp(uv, 0.0, 1.0);

    vec4 bigger = texture2D(u_Tex, uv);
    gray = grayscale(bigger, LUMINOSITY);
    col = rect != 0.0 ? color[int(mod(float(colIndex), 4.))] * gray : col;

    color[0] = color3;
    color[1] = color1;
    color[2] = color4;
    color[3] = color2;

    dim = vec2(1. / 2., 0.251 / 2.);
    center = vec2(dim.x, 1. - dim.y);
    tol = 0.0000001;
    rect = sdRect(1.0, v_Tex, center, dim, tol);

    scale = 3.825;
    uv = vec2(v_Tex.x, 1. - v_Tex.y) * scale;
    uv = f * (uv - 0.5) + 0.5;
    index = int(uv.x) + int(uv.y) * 4;
    colIndex = 7 - index + int(uv.y);
    if(int(mod(float(colIndex), 2.)) != 0) uv = 1. - uv;
    uv = fract(uv);
    uv = clamp(uv, 0.0, 1.0);

    bigger = texture2D(u_Tex, uv);
    gray = grayscale(bigger, LUMINOSITY);
    col = rect != 0.0 ? color[int(mod(float(colIndex), 4.))] * gray : col;

    gl_FragColor = col;

}
