#extension GL_OES_EGL_image_external : require
//precision mediump float;
varying highp vec4 vTextureCoord;
varying highp vec3 vTint;
varying highp vec3 vDots[8];

uniform samplerExternalOES sTexture;
uniform sampler2D sLut;
uniform highp float uBlend;
uniform highp vec2 uResolution;
uniform highp float uProgress;

highp vec4 applyLut(highp vec4 pixel);

void main() {
    highp vec3 c = texture2D (sTexture, vTextureCoord.xy).rgb;
    if (vTextureCoord.z > -0.025 && vTextureCoord.z < 1.025 && vTextureCoord.w > -0.025 && vTextureCoord.w < 1.025) {
        highp vec2 uv = vTextureCoord.xy * uResolution;
        highp vec3 b = vTint;

        if (vTextureCoord.z < 0.0) {
            b *= 1.0 - (vTextureCoord.z * -40.0);
        }

        if (vTextureCoord.z > 1.0) {
            b *= 1.0 - (vTextureCoord.z - 1.0) * 40.0;
        }

        if (vTextureCoord.w < 0.0) {
            b *= 1.0 - (vTextureCoord.w * -40.0);
        }

        if (vTextureCoord.w > 1.0) {
            b *= 1.0 - (vTextureCoord.w - 1.0) * 40.0;
        }

        c = c * (vec3 (1.0) - b) + b;

        for (int num = 0; num < 8; ++num) {
            if (distance (uv, vDots[num].xy) < vDots[num].z) {
                highp float dist = distance (uv, vDots[num].xy) / vDots[num].z;
                c *= 1.0 - (sin (1.0 - dist) * (vDots[num].z / 10.0));
            }
        }
    }

    gl_FragColor = applyLut(vec4 (c, 1.0));
}

highp vec4 applyLut(highp vec4 pixel) {
    if (uBlend <= 0.0) return pixel;

    if (pixel.x < 0.0) pixel.x = 0.0;
    if (pixel.y < 0.0) pixel.y = 0.0;
    if (pixel.z < 0.0) pixel.z = 0.0;

    if (pixel.x > 1.0) pixel.x = 1.0;
    if (pixel.y > 1.0) pixel.y = 1.0;
    if (pixel.z > 1.0) pixel.z = 1.0;

    highp vec2 look = vec2 (pixel.r * 31.0, pixel.g * 31.0);
    look.x += floor (mod (pixel.b * 31.0, 8.0)) * 32.0;
    look.y += floor (pixel.b * 3.875) * 32.0;
    look.x = (look.x + 0.5) / 256.0;
    look.y = (look.y + 0.5) / 128.0;

    highp vec4 hue = texture2D (sLut, look);
    highp float where = fract (pixel.b * 31.0);

    if (where > 0.003) {
        look.x += 0.125;

        if (look.x > 1.0) {
            look.y += 0.25;
            look.x -= 1.0;
        }

        highp vec4 up = texture2D (sLut, look);
        hue = mix (hue, up, where);
    }

    return mix (pixel, vec4 (hue.r, hue.g, hue.b, 1.0), uBlend);
}