uniform mat4 uMVPMatrix;
uniform mat4 uSTMatrix;
uniform highp vec2 uResolution;
uniform highp float uProgress;

attribute vec4 aPosition;
attribute vec4 aTextureCoord;

varying highp vec4 vTextureCoord;
varying highp vec3 vTint;
varying highp vec3 vDots[8];

const highp float a = 12.9898;
const highp float b = 78.233;
const highp float c = 43758.5453;

highp float random (vec2 p) {
    highp float dt = dot (p, vec2 (a, b));
    highp float sn = mod (dt, 3.14);
    return fract (sin (sn) * c);
}

vec3 noise3 (float x) {
    return vec3 (random (vec2 (x, 1.5)), random (vec2 (x, 5.0)), random (vec2 (x, 8.5)));
}

void main() {
    highp vec4 position = uMVPMatrix * aPosition;
    gl_Position = vec4 (position.x, position.y, 0.0, 1.0);

    highp vec3 tints[6];
    tints[0] = vec3 (0.5, 0.0, 0.0);
    tints[1] = vec3 (0.0, 0.0, 0.0);
    tints[2] = vec3 (1.0, 0.55, 0.2);
    tints[3] = vec3 (1.1, 0.55, 0.2);
    tints[4] = vec3 (1.0, 0.62, 0.28);
    tints[5] = vec3 (1.0, 1.0, 0.92);

    highp vec3 slices[6];
    slices[0] = vec3 (-0.84, -0.12, -0.25);
    slices[1] = vec3 (0.0, 0.0, 0.0);
    slices[2] = vec3 (0.56, -0.085, 0.05);
    slices[3] = vec3 (-0.48, 0.06, 0.0);
    slices[4] = vec3 (-0.32, 0.06, 0.0);
    slices[5] = vec3 (0.0, 0.0, -0.12);

    float frame = fract (uProgress) * 6.0;
    highp vec3 slice;

    if (frame < 1.0) {
        vTint = tints[0];
        slice = slices[0];
    } else if (frame < 2.0) {
        vTint = tints[1];
        slice = slices[1];
    } else if (frame < 3.0) {
        vTint = tints[2];
        slice = slices[2];
    } else if (frame < 4.0) {
        vTint = tints[3];
        slice = slices[3];
    } else if (frame < 5.0) {
        vTint = tints[4];
        slice = slices[4];
    } else {
        vTint = tints[5];
        slice = slices[5];
    }

    vTextureCoord.xy = (uSTMatrix * aTextureCoord).xy;
    vTextureCoord.z = (vTextureCoord.y - slice.x) + vTextureCoord.x * slice.y;
    vTextureCoord.w = vTextureCoord.x + slice.z;
    lowp vec2 grid = vec2 (uResolution.x / 4.0, uResolution.y / 2.0);

    for (int x = 0; x < 4; ++x) {
        for (int y = 0; y < 2; ++y) {
            vDots[x * 2 + y] = noise3 ((float (x * 2 + y) + uProgress) / 8.0);
            vDots[x * 2 + y].xy *= grid;
            vDots[x * 2 + y].xy += vec2 (float (x) * grid.x, float (y) * grid.y);
            vDots[x * 2 + y].z *= 4.0;
        }
    }

    //vTextureCoord = (uSTMatrix * aTextureCoord).xy;
}