#extension GL_OES_EGL_image_external : require
//precision mediump float;
varying highp vec2 vTextureCoord;
varying lowp vec3 vBlock;

uniform samplerExternalOES sTexture;
uniform sampler2D sLut;
uniform highp float uBlend;
uniform lowp float uProgress;
uniform highp vec2 uResolution;

highp vec4 applyLut(highp vec4 pixel);

void main() {
    highp vec2 texel = vec2 (floor (vTextureCoord.x / vBlock.x) * vBlock.x, floor (vTextureCoord.y / vBlock.y) * vBlock.y);
    highp vec4 color = vec4 (0.0);
    highp float scale;

    for (highp float y = 1.0; y < 96.0; y += 2.0) {
        if (y >= vBlock.y) {
            if (y - 0.5 > vBlock.y) {
                break;
            }

            texel.y -= 0.5;
            scale = 0.5;
        } else
        scale = 1.0;

        for (highp float x = 1.0; x < 96.0; x += 2.0) {
            if (x >= vBlock.x) {
                if (x - 0.5 < vBlock.x) {
                    mediump vec2 uv = vec2 (texel.x + x - 0.5, texel.y + y) / uResolution;
                    color += texture2D(sTexture, uv) * vBlock.z * scale * 0.5;
                }

                break;
            }

            mediump vec2 uv = vec2 (texel.x + x, texel.y + y) / uResolution;
            color += texture2D(sTexture, uv) * vBlock.z * scale;
        }
    }

    gl_FragColor = applyLut(color / (vBlock.x * vBlock.y * 0.25));
}

highp vec4 applyLut(highp vec4 pixel) {
    if (uBlend <= 0.0) return pixel;

    if (pixel.x < 0.0) pixel.x = 0.0;
    if (pixel.y < 0.0) pixel.y = 0.0;
    if (pixel.z < 0.0) pixel.z = 0.0;

    if (pixel.x > 1.0) pixel.x = 1.0;
    if (pixel.y > 1.0) pixel.y = 1.0;
    if (pixel.z > 1.0) pixel.z = 1.0;

    highp vec2 look = vec2 (pixel.r * 31.0, pixel.g * 31.0);
    look.x += floor (mod (pixel.b * 31.0, 8.0)) * 32.0;
    look.y += floor (pixel.b * 3.875) * 32.0;
    look.x = (look.x + 0.5) / 256.0;
    look.y = (look.y + 0.5) / 128.0;

    highp vec4 hue = texture2D (sLut, look);
    highp float where = fract (pixel.b * 31.0);

    if (where > 0.003) {
        look.x += 0.125;

        if (look.x > 1.0) {
            look.y += 0.25;
            look.x -= 1.0;
        }

        highp vec4 up = texture2D (sLut, look);
        hue = mix (hue, up, where);
    }

    return mix (pixel, vec4 (hue.r, hue.g, hue.b, 1.0), uBlend);
}