#extension GL_OES_EGL_image_external : require
precision mediump float;

varying vec2 vPosition;
varying vec2 vTextureCoord;

uniform samplerExternalOES sTexture;
uniform sampler2D sSecondTexture;
uniform sampler2D sLut;
uniform float uProgress;
uniform highp float uBlend;

highp vec4 applyLut(highp vec4 pixel);

highp vec4 colorForDelta(lowp float delta) {
    highp vec2 textureCoord = vTextureCoord;
    highp float two_pi = 2.0 * 3.14159265358979323846;
    highp float sin_factor = sin(two_pi * (uProgress + delta));
    highp float cos_factor = cos(two_pi * (uProgress + delta));
    textureCoord = (textureCoord - 0.5) * mat2(cos_factor, sin_factor, -sin_factor, cos_factor);
    textureCoord += 0.5;
    return mix(texture2D(sTexture, textureCoord), texture2D(sSecondTexture, textureCoord), uProgress);
}

void main() {
    gl_FragColor = applyLut((colorForDelta(-0.005) + colorForDelta(0.0) + colorForDelta(0.005)) / 3.0);
}

highp vec4 applyLut(highp vec4 pixel) {
    if (uBlend <= 0.0) return pixel;

    if (pixel.x < 0.0) pixel.x = 0.0;
    if (pixel.y < 0.0) pixel.y = 0.0;
    if (pixel.z < 0.0) pixel.z = 0.0;

    if (pixel.x > 1.0) pixel.x = 1.0;
    if (pixel.y > 1.0) pixel.y = 1.0;
    if (pixel.z > 1.0) pixel.z = 1.0;

    highp vec2 look = vec2 (pixel.r * 31.0, pixel.g * 31.0);
    look.x += floor (mod (pixel.b * 31.0, 8.0)) * 32.0;
    look.y += floor (pixel.b * 3.875) * 32.0;
    look.x = (look.x + 0.5) / 256.0;
    look.y = (look.y + 0.5) / 128.0;

    highp vec4 hue = texture2D (sLut, look);
    highp float where = fract (pixel.b * 31.0);

    if (where > 0.003) {
        look.x += 0.125;

        if (look.x > 1.0) {
            look.y += 0.25;
            look.x -= 1.0;
        }

        highp vec4 up = texture2D (sLut, look);
        hue = mix (hue, up, where);
    }

    return mix (pixel, vec4 (hue.r, hue.g, hue.b, 1.0), uBlend);
}