#extension GL_OES_EGL_image_external : require
precision mediump float;

varying vec2 vPosition;
varying vec2 vTextureCoord;

uniform samplerExternalOES sTexture;
uniform sampler2D sLut;
uniform float uProgress;
uniform highp float uBlend;

highp vec4 applyLut(highp vec4 pixel);

void main() {
    float pos = uProgress - 1.0;
    if (vPosition.y < pos || (vPosition.y > - pos)) {
        gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);
    } else {
        gl_FragColor = applyLut(texture2D(sTexture, vTextureCoord));
    }
}

highp vec4 applyLut(highp vec4 pixel) {
    if (uBlend <= 0.0) return pixel;

    if (pixel.x < 0.0) pixel.x = 0.0;
    if (pixel.y < 0.0) pixel.y = 0.0;
    if (pixel.z < 0.0) pixel.z = 0.0;

    if (pixel.x > 1.0) pixel.x = 1.0;
    if (pixel.y > 1.0) pixel.y = 1.0;
    if (pixel.z > 1.0) pixel.z = 1.0;

    highp vec2 look = vec2 (pixel.r * 31.0, pixel.g * 31.0);
    look.x += floor (mod (pixel.b * 31.0, 8.0)) * 32.0;
    look.y += floor (pixel.b * 3.875) * 32.0;
    look.x = (look.x + 0.5) / 256.0;
    look.y = (look.y + 0.5) / 128.0;

    highp vec4 hue = texture2D (sLut, look);
    highp float where = fract (pixel.b * 31.0);

    if (where > 0.003) {
        look.x += 0.125;

        if (look.x > 1.0) {
            look.y += 0.25;
            look.x -= 1.0;
        }

        highp vec4 up = texture2D (sLut, look);
        hue = mix (hue, up, where);
    }

    return mix (pixel, vec4 (hue.r, hue.g, hue.b, 1.0), uBlend);
}