#extension GL_OES_EGL_image_external : require
//precision mediump float;
varying highp vec2 vTextureCoord;
varying highp float vScale;
varying highp float vSpeed;

uniform samplerExternalOES sTexture;
uniform sampler2D sLut;
uniform highp float uBlend;
uniform highp vec2 uResolution;

highp vec4 applyLut(highp vec4 pixel);

void main() {
    highp vec2 org = (vTextureCoord - vec2 (0.5));
    highp vec2 dir = org * vSpeed * uResolution;
    highp float width = floor (length (dir) + 1.0);
    dir = normalize (dir) / uResolution * vScale;
    org *= vScale;
    highp vec3 c = vec3 (0);

    for (highp float x = 0.0; x < 256.0; x += 2.0) {
        if (x == width) break;

        if (width - x < 2.0) {
            highp vec2 uv = (org + dir * x) + vec2 (0.5);
            uv = abs (mod (uv + vec2 (1.0), 2.0) - vec2 (1.0));
            c += texture2D(sTexture, uv).rgb;
            break;
        }

        highp vec2 uv = (org + dir * (x + 0.5)) + vec2 (0.5);
        uv = abs (mod (uv + vec2 (1.0), 2.0) - vec2 (1.0));
        c += texture2D(sTexture, uv).rgb * 2.0;
    }

    gl_FragColor = applyLut(vec4 (c / width, 1.0));
    //gl_FragColor = applyLut(texture2D(sTexture, vTextureCoord));
}

highp vec4 applyLut(highp vec4 pixel) {
    if (uBlend <= 0.0) return pixel;

    if (pixel.x < 0.0) pixel.x = 0.0;
    if (pixel.y < 0.0) pixel.y = 0.0;
    if (pixel.z < 0.0) pixel.z = 0.0;

    if (pixel.x > 1.0) pixel.x = 1.0;
    if (pixel.y > 1.0) pixel.y = 1.0;
    if (pixel.z > 1.0) pixel.z = 1.0;

    highp vec2 look = vec2 (pixel.r * 31.0, pixel.g * 31.0);
    look.x += floor (mod (pixel.b * 31.0, 8.0)) * 32.0;
    look.y += floor (pixel.b * 3.875) * 32.0;
    look.x = (look.x + 0.5) / 256.0;
    look.y = (look.y + 0.5) / 128.0;

    highp vec4 hue = texture2D (sLut, look);
    highp float where = fract (pixel.b * 31.0);

    if (where > 0.003) {
        look.x += 0.125;

        if (look.x > 1.0) {
            look.y += 0.25;
            look.x -= 1.0;
        }

        highp vec4 up = texture2D (sLut, look);
        hue = mix (hue, up, where);
    }

    return mix (pixel, vec4 (hue.r, hue.g, hue.b, 1.0), uBlend);
}